/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Billboard
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        float Rot = 0;
        Matrix rotMatrix = Matrix.Identity;
        Model tiny;
        Model box;
        bool billboard = false;
        float camdist = 400;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            this.IsMouseVisible = true;
            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                tiny = content.Load<Model>(@".\Meshes\tiny");
                box = content.Load<Model>(@".\Meshes\Box");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            if (Keyboard.GetState().IsKeyDown(Keys.Escape)) this.Exit();

            // hold spacebar to use billboarding
            if (Keyboard.GetState().IsKeyDown(Keys.Space) && billboard == false) billboard = true;
            if (Keyboard.GetState().IsKeyUp(Keys.Space) && billboard == true) billboard = false;

            // get rotation from mouse
            Rot = Mouse.GetState().X * 0.01f;

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // setup view and projection matrixes
            Matrix view = Matrix.CreateLookAt(GetCamPos(), Vector3.Zero, Vector3.Up);
            float aspect = this.Window.ClientBounds.Width / this.Window.ClientBounds.Height;
            Matrix proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.Pi * 0.5f, aspect, 1, 10000);
            // create a billboard matrix that tiny well use to always face the camera
            if (billboard)
            {
                // create billboard matrix
                rotMatrix = Matrix.CreateBillboard(Vector3.Zero, GetCamPos(), Vector3.Up, Vector3.Zero);

                // uncomment to restrict along up axis (same as camera up vector)
                //rotMatrix = Matrix.CreateConstrainedBillboard(Vector3.Zero, GetCamPos(), Vector3.Up, Vector3.Zero, GetCamPos() + new Vector3(100, 0, 0));

                // uncomment to restrict along left axis
                //rotMatrix = Matrix.CreateConstrainedBillboard(Vector3.Zero, GetCamPos(), Vector3.Left, Vector3.Zero, GetCamPos() + new Vector3(100, 0, 0));
            }
            else
            {
                // no billboarding will be used
                rotMatrix = Matrix.Identity;
            }

            // this matrix stores models position/rotation/scale info etc
            Matrix pos = Matrix.Identity;

            // draw tiny mesh
            DrawMesh(tiny, view, proj, pos * rotMatrix);
            // size the box and position it, then draw
            pos *= Matrix.CreateTranslation(0, -50, 0) * Matrix.CreateScale(10);
            DrawMesh(box, view, proj, pos);

            base.Draw(gameTime);
        }

        private void DrawMesh(Model m, Matrix view, Matrix proj, Matrix pos)
        {
            foreach (ModelMesh mesh in m.Meshes)
            {
                foreach (BasicEffect fx in mesh.Effects)
                {
                    fx.View = view;
                    fx.Projection = proj;
                    fx.World = pos;
                }
                mesh.Draw();
            }
        }

        Vector3 GetCamPos()
        {
            return new Vector3((float)Math.Cos(Rot) * camdist, 0, (float)Math.Sin(Rot) * camdist);
        }
    }
}